package com.factory.init;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Callable;
import java.util.concurrent.ConcurrentMap;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;

import org.quartz.DisallowConcurrentExecution;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.factory.base.entity.task.FirstCalendar;
import com.factory.base.entity.task.FirstTask;
import com.factory.common.CommonUtil;
import com.factory.service.FirstTaskService;

/**
 * 首件任务定时器
 */
@DisallowConcurrentExecution
public class FirstTaskJob implements Job {
    private static Logger log = LoggerFactory.getLogger(FirstTaskJob.class);

    @Override
    public void execute(JobExecutionContext context) {
        ConcurrentMap<String, Object> paramMap = (ConcurrentMap<String, Object>) context.getJobDetail().getJobDataMap().get(QuartzManager.JOB_PARAM_NAME);
        try {
            createTask(paramMap);// 执行任务
        } catch (Throwable e) {
        	log.error("execute生产任务出错",e);
        }
    }

    /**
     * 创建Task主方法
     *
     * @param paramMap
     */
    public void createTask(ConcurrentMap<String, Object> paramMap) {
    	ExecutorService executorService = (ExecutorService) paramMap.get("executorService");
        List<Callable<Integer>> tasks = new ArrayList<>();
        tasks.add(getFirstFutureTask(paramMap));
        tasks.add(get200FutureTask(paramMap));
        try {
        	List<Future<Integer>> rlist = executorService.invokeAll(tasks);
            for (Future<Integer> t : rlist) {
            	t.get();
            }//等等所有线程结束
        } catch (Throwable e) {
        	log.error("createTask生产任务出错",e);
        }

    }

    /**
     * day
     *
     * @param paramMap
     * @return
     */
    private Callable<Integer> getFirstFutureTask(ConcurrentMap<String, Object> paramMap) {
    	Callable<Integer> task = new AddTaskWorker(paramMap, 0);
        return task;
    }

    /**
     * 200
     *
     * @param paramMap
     * @return
     */
    private Callable<Integer> get200FutureTask(ConcurrentMap<String, Object> paramMap) {
    	Callable<Integer> task = new AddTaskWorker(paramMap, 1);
        return task;
    }


    /**
     * 生产任务
     */
    public class AddTaskWorker implements Callable<Integer> {

        private ConcurrentMap<String, Object> paramMap;
        private Integer typeFlag;


        public AddTaskWorker(ConcurrentMap<String, Object> paramMap, Integer typeFlag) {
            this.paramMap = paramMap;
            this.typeFlag = typeFlag;
        }

        @Override
        public Integer call() {
            FirstTaskService firstTaskService = (FirstTaskService) paramMap.get("firstTaskService");
            LocalDate now = LocalDate.now();
            Map<String,Object> param = new HashMap<>();
            param.put("typeFlag", typeFlag);
            param.put("date", now);
            List<FirstCalendar> cals = firstTaskService.getCalenderFirstForType(param);
            for (FirstCalendar cal : cals) {
                try {
                    // TODO 添加不同类型添加一个task代码
                    FirstTask task = new FirstTask();
                    task.setTask_time(now);
                    task.setFirst_article_calendar_id(cal.getId());
                    task.setType_flag(typeFlag);
                    firstTaskService.insertFirstTask(task);
                } catch (Throwable e) {
                    log.error("addTask生产任务出错",e);
                }
            }
            return 0;
        }
    }

}
