package com.factory.service.impl;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.factory.base.entity.task.ApprovalUserRes;
import com.factory.base.entity.task.ApproveEntry;
import com.factory.base.entity.task.FirstCalendar;
import com.factory.base.entity.task.FirstTask;
import com.factory.base.entity.task.SendMsgReq;
import com.factory.mapper.FirstTaskMapper;
import com.factory.service.FirstTaskService;
import com.factory.service.MsgService;

@Service
public class FirstTaskServiceImp implements FirstTaskService {

    @Autowired
    FirstTaskMapper firstTaskMapper;
    @Autowired
    MsgService msgService;

    /**
     * 获取对应的规则
     *
     * @param typeFlag
     * @return
     */
    @Override
    public List<FirstCalendar> getCalenderFirstForType(Map<String, Object> param) {
        return firstTaskMapper.getCalenderFirstForType(param);
    }

    /**
     * 添加当天任务
     *
     * @param task
     * @return
     */
    @Override
    public int insertFirstTask(FirstTask task) {
    	firstTaskMapper.insertFirstTaskKey(task);
        return firstTaskMapper.insertFirstTask(task);
    }

    /**
     * 需要提醒的工作
     *
     * @param param
     * @return
     */
    @Override
    public List<FirstCalendar> getRemindWork(Map<String, Object> param) {
        return firstTaskMapper.getRemindWork(param);
    }


    /**
     * 更新提醒状态，不太好的写法但是符合现在的需求
     *
     * @param param
     * @return
     */
    @Override
    public int updateFirstRemindFlag(Map<String, Object> param) {
        return firstTaskMapper.updateFirstRemindFlag(param);
    }

    /**
     * 添加不工作提醒信息
     *
     * @param map
     * @return
     */
    @Override
    @Transactional(rollbackFor = Throwable.class)
    public int addRemindNowork(Map<String, Object> map) {
        int result = firstTaskMapper.addRemindNowork(map);
        firstTaskMapper.updateFirstRemindFlag(map);
        return result;
    }

    /**
     * 获取需要提醒人
     *
     * @param map
     * @return
     */
    @Override
    public Long getRemindNoworkToManager(Map<String, Object> map) {
        return firstTaskMapper.getRemindNoworkToManager(map);
    }


    /**
     * 获取最大审批级别
     *
     * @param map
     * @return
     */
    @Override
    public int getMaxApproveLevel(Map<String, Object> map) {
        return firstTaskMapper.getMaxApproveLevel(map);
    }

    /**
     * 或许超时没处理记录
     *
     * @param map
     * @return
     */
    @Override
    public List<ApproveEntry> getApproveTaskNoDeal(Map<String, Object> map) {
        return firstTaskMapper.getApproveTaskNoDeal(map);
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public int addTaskForNext(ApproveEntry vo) {
        HashMap<String,Object> map = new HashMap<>();
        map.put("approvalLevel", vo.getApproval_level());
        vo.setStatus(2);
        vo.setApproval_user_id(1l);
        ApprovalUserRes nextUser = firstTaskMapper.getApprovalNextUser(map);
        if (nextUser != null) {
            int result = firstTaskMapper.updateApproveTask(vo);
            if (result > 0) {
                ApproveEntry approve = new ApproveEntry();
                approve.setTask_id(vo.getTask_id());
                approve.setApproval_level(nextUser.getApproval_level());
                approve.setApproval_role(nextUser.getRole_id());
                
                int status = firstTaskMapper.addApproveTask(approve);
                if (status > 0) {
                    Date date = new Date();
                    SimpleDateFormat ft = new SimpleDateFormat("yyyy-MM-dd");
                    //发送微信消息
                    SendMsgReq sendMsgReq = new SendMsgReq();
                    sendMsgReq.setToUserId(nextUser.getUser_id());
                    sendMsgReq.setTitle("计划放行提醒");
                    sendMsgReq.setContent("您有待放行的计划审批，请登录小程序查看【计划放行】");
                    sendMsgReq.setEndTime(ft.format(date));
                    msgService.sendWechatMessageByRole(sendMsgReq,nextUser.getRole_id());
                }
                return status;
            }
        }
        return 0;
    }


}
